/*
Copyright (C) 2011 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
 */
package org.harctoolbox.IrpMaster.Iterate;

import org.harctoolbox.IrpMaster.IrpUtils;

/**
 * A value set is either an single value or a way
 * of generating repetitions. These are implemented in subclasses.
 * 
 */
public abstract class ValueSet implements Iterable<Long> {
    protected long min;
    protected long current; // Memory for the iterator

    protected static String formatThing(String prefix, int variable) {
        return (variable == IrpUtils.invalid) ? "" : (prefix + "=" + variable);
    }

    protected ValueSet(long min) {
        this.min = min;
        current = IrpUtils.invalid;
    }

    /**
     * Resets the iterator to virgin state.
     */
    public void reset() {
        current = IrpUtils.invalid;
    }

    public static ValueSet newValueSet(String s) {
        if (s.contains("<<")) {
            String[] q = s.split("<<");
            long min = IrpUtils.parseLong(q[0]);
            if (min == 0)
                throw new IllegalArgumentException("Shifting 0 in " + s + " senseless.");
            long max = IrpUtils.parseUpper(q[0]);
            int shift = (int) IrpUtils.parseLong(q[1]);
            return new ShiftValueSet(min, max, shift);
        } else if (s.contains("++")) {
            String[] q = s.split("\\+\\+");
            long min = IrpUtils.parseLong(q[0]);
            long max = IrpUtils.parseUpper(q[0]);
            int increment = (int) IrpUtils.parseLong(q[1]);
            return new IterationValueSet(min, max, increment);
        } else if (s.contains("#")) {
            String[] q = s.split("#");
            long min = IrpUtils.parseLong(q[0]);
            long max = IrpUtils.parseUpper(q[0]);
            int noRandoms = (int) IrpUtils.parseLong(q[1]);
            return new RandomValueSet(min, max, noRandoms);
        } else {
            long min = IrpUtils.parseLong(s);
            long max = IrpUtils.parseUpper(s);
            return (max == IrpUtils.invalid) ? new SingletonValueSet(min) : new IterationValueSet(min, max, 1);
        }
    }

    /**
     * Just for testing purposes.
     * 
     * @param args
     */
    public static void main(String[] args) {
        int seed = (int) IrpUtils.invalid;
        int arg_i = 0;
        if (args[arg_i].equals("-s")) {
            seed = Integer.parseInt(args[++arg_i]);
            arg_i++;
        }

        RandomValueSet.initRng(seed);

        ValueSet vs = newValueSet(args[arg_i]);
        System.out.println(vs);
        System.out.println();

        for (Long l : vs) {
            System.out.println(l);
        }
    }
}
